BFieldElement

Struct BFieldElement 

pub struct BFieldElement(/* private fields */);
Base field element ∈ ℤ_{2^64 - 2^32 + 1}.

In Montgomery representation. This implementation follows https://eprint.iacr.org/2022/274.pdf and https://github.com/novifinancial/winterfell/pull/101/files.

Implementations

Source

impl BFieldElement

pub const BYTES: usize = 8

pub const P: u64 = 0xffff_ffff_0000_0001

The base field’s prime, i.e., 2^64 - 2^32 + 1.

pub const MAX: u64

pub const MINUS_TWO_INVERSE: Self

-2^-1

pub const fn new(value: u64) -> Self

pub const fn value(&self) -> u64

pub fn inverse(&self) -> Self

pub const fn power_accumulator<const N: usize, const M: usize>( base: [Self; N], tail: [Self; N], ) -> [Self; N]

Square the base M times and multiply the result by the tail value

pub const fn generator() -> Self

A generator for the entire base field.

pub const fn lift(&self) -> XFieldElement

Turn this base field element into the corresponding extension field element.

pub fn increment(&mut self)

Add BFieldElement::ONE to self.

pub fn decrement(&mut self)

Subtract BFieldElement::ONE from self.

pub const fn mod_pow(&self, exp: u64) -> Self

pub const fn montyred(x: u128) -> u64

Montgomery reduction

pub const fn raw_bytes(&self) -> [u8; 8]

Return the raw bytes or 8-bit chunks of the Montgomery representation, in little-endian byte order

pub const fn from_raw_bytes(bytes: &[u8; 8]) -> Self

Take a slice of 8 bytes and interpret it as an integer in little-endian byte order, and cast it to a BFieldElement in Montgomery representation

pub const fn raw_u16s(&self) -> [u16; 4]

Return the raw 16-bit chunks of the Montgomery representation, in little-endian chunk order

pub const fn from_raw_u16s(chunks: &[u16; 4]) -> Self

Take a slice of 4 16-bit chunks and interpret it as an integer in little-endian chunk order, and cast it to a BFieldElement in Montgomery representation

pub fn raw_u128(&self) -> u128

pub const fn from_raw_u64(e: u64) -> BFieldElement

pub const fn raw_u64(&self) -> u64

pub const fn is_canonical(x: u64) -> bool