pub enum Domain {
VariableLength,
FixedLength,
}Expand description
The hasher Domain differentiates between the modes of hashing.
The main purpose of declaring the domain is to prevent collisions between different types of hashing by introducing defining differences in the way the hash function’s internal state (e.g. a sponge state’s capacity) is initialized.
Variants
VariableLength
The VariableLength domain is used for hashing objects that potentially serialize to more
than RATE number of field elements.
FixedLength